import yaml
from typing import Literal, Any, Optional, Mapping
from theia import Theia
from theia.aws.network.vpc import Vpc, VpcSettings

with open ('input.yaml','r') as f:
    defaultInputs = yaml.safe_load(f)

def getProject(userinputs: dict):
    return f"{userinputs['project']}-{userinputs['env']}"

class testbase:

    def getBaseResourceName(self):
        resourceList = {}
        vpc = {"vpc1" : f"{self.getStackName()}-vpc"}
        resourceList.update(vpc)
        return resourceList
    
    def getStackName(self):
        stack = f"{self.userInputs['project']}-{self.userInputs['env']}-{self.stack}"
        return stack
    
    def __init__(self,
                 userInputs: dict,
                 defaultInputs: Optional[dict] = defaultInputs):
        
        self.userInputs = userInputs
        self.defaultInputs = defaultInputs

        self.stack = testbase.__name__
        
        self.project = getProject(self.userInputs)
        self.region = self.userInputs['region']
        self.email = self.userInputs['email']
        self.backend_url = self.userInputs['backend_url']
        self.vpc_ipv4_cidr = self.userInputs['ipv4_cidr']

        self.defaultInputs['tags']['created_by'] = f"{self.email}"
        resourceTags = {"Stack" : f"{self.getStackName()}"}
        self.defaultInputs['tags'].update(resourceTags)
        self.defaultInputs['tags'].update(self.userInputs['tags'])
        self.tags = self.defaultInputs['tags']


    def infra(self):

        resourceList = self.getBaseResourceName()

        def program():
            vpcName = resourceList['vpc1']
            vpcCidr = self.vpc_ipv4_cidr
            vpcTags = self.tags

            vpc = Vpc(resource_name=vpcName,
                      ipv4_cidr=vpcCidr,
                      tenancy="default",
                      settings=VpcSettings(),
                      tags=vpcTags)
            
        t =  Theia(project=self.project,
                    stack=self.stack,
                    program=program,
                    region=self.region,
                    backend_url=self.backend_url)
            
        return t
        
    def __call__(self, command: Literal['preview', 'up', 'destroy', 'refresh']) -> Any:
        t = self.infra()
        getattr(t, command)()